from dataclasses import dataclass
from pathlib import Path
import pandas as pd
import argparse


class Constants:
    START_HEADER = 0

    class PolyOpinion:
        firstname: str = "firstname"
        lastname: str = "lastname"
        email: str = "email"

    class Poly:
        name: str = "Nom de l'étudiant"
        email: str = "Adresse courriel poly"


@dataclass(frozen=True)
class Arguments:
    file: Path
    sheet: int
    output: Path
    program: str | None = None


def get_args() -> Arguments:
    parser = argparse.ArgumentParser(
        description="Converts AEP members Excel file to PolyOpinion csv"
    )

    parser.add_argument(
        "--file",
        "-f",
        type=Path,
        help="Path to the excel file to convert",
        required=True,
    )

    parser.add_argument(
        "--sheet",
        "-s",
        type=int,
        help="Sheet number to convert",
        required=True,
    )

    parser.add_argument(
        "--output",
        "-o",
        type=Path,
        help="Path to the output csv file",
        required=True,
    )

    parser.add_argument(
        "--program",
        "-p",
        type=str,
        help=(
            "Filter the student program. "
            "Must be a value in the 'Prog.' column of the Excel file."
        ),
        required=False,
    )

    args = parser.parse_args()

    return Arguments(
        file=args.file,
        sheet=args.sheet,
        output=args.output,
        program=args.program,
    )


def main() -> None:
    args = get_args()

    df = pd.read_excel(
        args.file,
        header=Constants.START_HEADER,
        sheet_name=args.sheet - 1,
    )

    # only keep values in the 'Prog.' column that are in args.filter
    if args.program:
        print(f"Filtering by program: {args.program}")
        df = df[df["Prog."] == args.program]

    print("Original data:")
    print(df.head())

    names = df[Constants.Poly.name].str.split(", ").str
    df = df.drop(columns=[Constants.Poly.name])

    polyop_df = pd.DataFrame()

    polyop_df[Constants.PolyOpinion.firstname] = names[1]
    polyop_df[Constants.PolyOpinion.lastname] = names[0]
    polyop_df[Constants.PolyOpinion.email] = df[Constants.Poly.email]

    print("Formatted data:")
    print(polyop_df.head())

    polyop_df.to_csv(args.output, index=False)


if __name__ == "__main__":
    main()
